#include <stdlib.h>
#include <string.h>

#include <gtk/gtk.h>
#include <gdk/gdkkeysyms.h>

#include "guiutils.h"
#include "fprompt.h"

#include "images/icon_ok_20x20.xpm"
#include "images/icon_cancel_20x20.xpm"
#include "images/icon_browse_20x20.xpm"


/*
 *	Floating Prompt:
 */
typedef struct _fprompt_struct		fprompt_struct;
struct _fprompt_struct {

	gboolean	initialized;

	gint		width,		/* Size of toplevel */
			height;

	GtkWidget       *toplevel,	/* Popup GtkWindow */
			*main_hbox,
			*label,
			*entry,
			*browse_btn,
			*ok_btn,
			*cancel_btn;

	gpointer	client_data;
	gchar	*(*browse_cb)(
		gpointer,		/* Data */
		const gchar *		/* Value */
	);
	void	(*apply_cb)(
		gpointer,		/* Data */
		const gchar *		/* Value */
	);
	void	(*cancel_cb)(
		gpointer		/* Data */
	);

};
#define FPROMPT(p)	((fprompt_struct *)(p))


static fprompt_struct fprompt;


static gint FPromptEventCB(
	GtkWidget *widget, GdkEvent *event, gpointer data
);
static gint FPromptDeleteEventCB(
	GtkWidget *widget, GdkEvent *event, gpointer data
);
static void FPromptBrowseCB(GtkWidget *widget, gpointer data);
static void FPromptOKCB(GtkWidget *widget, gpointer data);
static void FPromptCancelCB(GtkWidget *widget, gpointer data);


gint FPromptInit(void);
void FPromptSetTransientFor(GtkWidget *w);
gboolean FPromptIsQuery(void);
void FPromptBreakQuery(void);
void FPromptSetPosition(gint x, gint y);
gint FPromptMapQuery(
	const gchar *label,
	const gchar *value,
	const gchar *tooltip_message,
	fprompt_map_code map_code,	/* One of FPROMPT_MAP_* */
	gint width, gint height,
	fprompt_flags flags,		/* Any of FPROMPT_FLAG_* */
	gpointer client_data,
	gchar *(*browse_cb)(gpointer, const gchar *),
	void (*apply_cb)(gpointer, const gchar *),
	void (*cancel_cb)(gpointer)
);
void FPromptMap(void);
void FPromptUnmap(void);
void FPromptShutdown(void);


#define ATOI(s)         (((s) != NULL) ? atoi(s) : 0)
#define ATOL(s)         (((s) != NULL) ? atol(s) : 0)
#define ATOF(s)         (((s) != NULL) ? atof(s) : 0.0f)
#define STRDUP(s)       (((s) != NULL) ? g_strdup(s) : NULL)

#define MAX(a,b)        (((a) > (b)) ? (a) : (b))
#define MIN(a,b)        (((a) < (b)) ? (a) : (b))
#define CLIP(a,l,h)     (MIN(MAX((a),(l)),(h)))
#define STRLEN(s)       (((s) != NULL) ? strlen(s) : 0)
#define STRISEMPTY(s)   (((s) != NULL) ? (*(s) == '\0') : TRUE)


/*
 *	Floating Prompt GtkEntry event signal callback.
 */
static gint FPromptEventCB(
	GtkWidget *widget, GdkEvent *event, gpointer data
)
{
	gint status = FALSE;
	gint etype;
	gboolean press;
	GdkEventFocus *focus;
	GdkEventKey *key;
	GdkEventButton *button;
	GdkEventMotion *motion;
	GdkEventCrossing *crossing;
	GtkWidget *w;
	fprompt_struct *p = FPROMPT(data);
	if((widget == NULL) || (event == NULL) || (p == NULL))
	    return(status);

	if(!FPromptIsQuery())
	    return(status);

	/* Handle by event type */
	etype = (gint)event->type;
	switch(etype)
	{
	  case GDK_FOCUS_CHANGE:
	    /* Focus events are actually never sent so handling them
	     * is pointless
	     */
	    focus = (GdkEventFocus *)event;
	    if(focus->in && !GTK_WIDGET_HAS_FOCUS(widget))
		GTK_WIDGET_SET_FLAGS(widget, GTK_HAS_FOCUS);
	    else if(!focus->in && GTK_WIDGET_HAS_FOCUS(widget))
		GTK_WIDGET_UNSET_FLAGS(widget, GTK_HAS_FOCUS);
	    status = TRUE;
	    break;

	  case GDK_KEY_PRESS:
	  case GDK_KEY_RELEASE:
	    key = (GdkEventKey *)event;
	    press = (etype == GDK_KEY_PRESS) ? TRUE : FALSE;
	    switch(key->keyval)
	    {
	      case GDK_Escape:
		if(press)
		{
		    /* Break query on escape, then return immediately
		     * so that the widget is not grabbed any more
		     */
		    FPromptBreakQuery();
		    status = TRUE;
		    return(status);
		}
		status = TRUE;
		break;

	      case GDK_Return:
	      case GDK_KP_Enter:
	      case GDK_ISO_Enter:
	      case GDK_3270_Enter:
		if(press)
		{
		    /* Call ok callback and return immediately so that
		     * the widget is not grabbed any more
		     */
		    FPromptOKCB(NULL, data);
		    status = TRUE;
		    return(status);
		}
		status = TRUE;
		break;
	    }
	    break;

	  case GDK_BUTTON_PRESS:
	    button = (GdkEventButton *)event;
	    w = p->toplevel;
	    if(w != NULL)
	    {
		gint x, y;
		GdkModifierType mask;
		GdkWindow *window = w->window;

		/* Clicked outside of the toplevel window? Check if the
		 * event GdkWindow differs from the toplevel GdkWindow
		 */
		gdk_window_get_pointer(window, &x, &y, &mask);
		if((x < 0) || (x >= p->width) ||
		   (y < 0) || (y >= p->height) 
		)
		{
		    /* Since a button event occured outside of the
		     * Floating Prompt's window we need to break query
		     * and return immediately so that the widget is not
		     * grabbed any more
		     */
		    FPromptBreakQuery();
		    status = TRUE;
		    return(status);
		}  
	    }
	    status = TRUE;
	    break;

	  case GDK_BUTTON_RELEASE:
	    button = (GdkEventButton *)event;
	    status = TRUE;
	    break;

	  case GDK_MOTION_NOTIFY:
	    motion = (GdkEventMotion *)event;
	    break;

	  case GDK_ENTER_NOTIFY:
	    crossing = (GdkEventCrossing *)event;
	    break;

	  case GDK_LEAVE_NOTIFY:
	    crossing = (GdkEventCrossing *)event;
	    break;

	}

	/* Need to regrab the widget, this is so that the next event
	 * such as a "key_press_event" is sent to this widget
	 */
	w = p->entry;
	if((w != NULL) && (w != gtk_grab_get_current()))
	    gtk_grab_add(w);

	return(status);
}

/*
 *	Toplevel GtkWindow "delete_event" signal callback.
 */
static gint FPromptDeleteEventCB(
	GtkWidget *widget, GdkEvent *event, gpointer data
)
{
	FPromptCancelCB(widget, data);
	return(TRUE);
}

/*
 *	Browse callback.
 */
static void FPromptBrowseCB(GtkWidget *widget, gpointer data)
{
	fprompt_struct *p = FPROMPT(data);
	if((widget == NULL) || (p == NULL))
	    return;  

	/* Call browse callback? */
	if(p->browse_cb != NULL)
	{
	    GtkWidget *w = p->entry;

	    /* Get current value */
	    gchar *v = STRDUP((w != NULL) ?
		gtk_entry_get_text(GTK_ENTRY(w)) : NULL
	    );

	    /* Call browse callback and get a statically allocated
	     * return string
	     */
	    const gchar *rtn_v = p->browse_cb(
		p->client_data,		/* Data */
		v			/* Value */
	    );
	    g_free(v);

	    /* Set new value obtained from the browse callback */
	    if((w != NULL) && (rtn_v != NULL))
		gtk_entry_set_text(GTK_ENTRY(w), rtn_v);
	}
}

/*
 *	OK callback.
 */
static void FPromptOKCB(GtkWidget *widget, gpointer data)
{
	fprompt_struct *p = FPROMPT(data);
	if(p == NULL)
	    return;

	/* Call apply callback? */
	if(p->apply_cb != NULL)
	{
	    GtkWidget *w = p->entry;
	    gchar *v = STRDUP((w != NULL) ?
		gtk_entry_get_text(GTK_ENTRY(w)) : NULL
	    );
	    if(v != NULL)
	    {
		/* Call apply callback */
		p->apply_cb(
		    p->client_data,	/* Data */
		    v			/* Value */
		);
		g_free(v);
	    }
	    p->apply_cb = NULL;
	    p->client_data = NULL;
	}

	FPromptUnmap();
}

/*
 *	Cancel callback.
 */
static void FPromptCancelCB(GtkWidget *widget, gpointer data)
{
	fprompt_struct *p = FPROMPT(data);
	if(p == NULL)
	    return;

	/* Call cancel callback? */
	if(p->cancel_cb != NULL)
	{
	    /* Call cancel callback */
	    p->cancel_cb(p->client_data);
	    p->cancel_cb = NULL;
	    p->client_data = NULL;
	}

	FPromptUnmap();
}


/*
 *	Initializes the Floating Prompt.
 */
gint FPromptInit(void)
{
	const gint border_minor = 2;
	GtkWidget *w, *parent;
	GdkWindow *window;
	fprompt_struct *p = &fprompt;

	/* Reset values */
	memset(p, 0x00, sizeof(fprompt_struct));
	p->initialized = TRUE;

	/* Begin creating widgets */

	/* Toplevel */
	p->toplevel = w = gtk_window_new(GTK_WINDOW_POPUP);
	gtk_widget_realize(w);                                    
	window = w->window;
	if(window != NULL)
	{
	    /* No decorations */
	    gdk_window_set_decorations(window, 0);
	    /* No functions */
	    gdk_window_set_functions(window, 0);

	    gdk_window_set_override_redirect(window, TRUE);
	}
	gtk_widget_set_usize(w, FPROMPT_DEF_WIDTH, FPROMPT_DEF_HEIGHT);
	gtk_signal_connect(
	    GTK_OBJECT(w), "delete_event",
	    GTK_SIGNAL_FUNC(FPromptDeleteEventCB), p
	);
	gtk_container_border_width(GTK_CONTAINER(w), 0);
	parent = w;

	/* Main outer frame */
	w = gtk_frame_new(NULL);
	gtk_frame_set_shadow_type(GTK_FRAME(w), GTK_SHADOW_OUT);
	gtk_container_add(GTK_CONTAINER(parent), w);
	gtk_widget_show(w);
	parent = w;

	/* Main hbox */
	p->main_hbox = w = gtk_hbox_new(FALSE, border_minor);
	gtk_container_add(GTK_CONTAINER(parent), w);
	gtk_widget_show(w);
	parent = w;

	/* Label */
	p->label = w = gtk_label_new("Value:");
	gtk_label_set_justify(GTK_LABEL(w), GTK_JUSTIFY_RIGHT);
	gtk_box_pack_start(GTK_BOX(parent), w, FALSE, FALSE, 0);
	gtk_widget_show(w);

	/* Entry */
	p->entry = w = gtk_entry_new();
	GTK_WIDGET_SET_FLAGS(w, GTK_CAN_DEFAULT | GTK_CAN_FOCUS);
	/* When connecting signals it is important to connect certain
	 * signals after in order for the widget to to be grabbed
	 * again in the event callback function
	 *
	 * Focus events are actually never sent so handling them is
	 * pointless
	 */
	gtk_widget_add_events(
	    w,
	    GDK_FOCUS_CHANGE_MASK |
	    GDK_KEY_PRESS_MASK | GDK_KEY_RELEASE_MASK |
	    GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK |
	    GDK_POINTER_MOTION_MASK | GDK_POINTER_MOTION_HINT_MASK |
	    GDK_ENTER_NOTIFY_MASK | GDK_LEAVE_NOTIFY_MASK
	);
	gtk_signal_connect_after(
	    GTK_OBJECT(w), "focus_in_event",   
	    GTK_SIGNAL_FUNC(FPromptEventCB), p
	);
	gtk_signal_connect_after(
	    GTK_OBJECT(w), "focus_out_event",
	    GTK_SIGNAL_FUNC(FPromptEventCB), p
	);
	gtk_signal_connect(
	    GTK_OBJECT(w), "key_press_event",
	    GTK_SIGNAL_FUNC(FPromptEventCB), p  
	);
	gtk_signal_connect(
	    GTK_OBJECT(w), "key_release_event",
	    GTK_SIGNAL_FUNC(FPromptEventCB), p
	);
	gtk_signal_connect(
	    GTK_OBJECT(w), "button_press_event",
	    GTK_SIGNAL_FUNC(FPromptEventCB), p
	);
	gtk_signal_connect_after(
	    GTK_OBJECT(w), "button_release_event",
	    GTK_SIGNAL_FUNC(FPromptEventCB), p
	);
	gtk_signal_connect_after(
	    GTK_OBJECT(w), "motion_notify_event",
	    GTK_SIGNAL_FUNC(FPromptEventCB), p
	);
	gtk_signal_connect_after(
	    GTK_OBJECT(w), "enter_notify_event",
	    GTK_SIGNAL_FUNC(FPromptEventCB), p
	);
	gtk_signal_connect_after(
	    GTK_OBJECT(w), "leave_notify_event",
	    GTK_SIGNAL_FUNC(FPromptEventCB), p 
	);
	gtk_box_pack_start(GTK_BOX(parent), w, TRUE, TRUE, 0);
	gtk_widget_show(w);

	/* Browse button */
	p->browse_btn = w = GUIButtonPixmap(
	    (guint8 **)icon_browse_20x20_xpm
	);
	gtk_box_pack_start(GTK_BOX(parent), w, FALSE, FALSE, 0);
	gtk_signal_connect(
	    GTK_OBJECT(w), "clicked",
	    GTK_SIGNAL_FUNC(FPromptBrowseCB), p
	);

	/* OK button */
	p->ok_btn = w = (GtkWidget *)GUIButtonPixmap(
	    (guint8 **)icon_ok_20x20_xpm 
	);
	gtk_box_pack_start(GTK_BOX(parent), w, FALSE, FALSE, 0);
	gtk_signal_connect(
	    GTK_OBJECT(w), "clicked",
	    GTK_SIGNAL_FUNC(FPromptOKCB), p
	);

	/* Cancel button */   
	p->cancel_btn = w = (GtkWidget *)GUIButtonPixmap(
	    (guint8 **)icon_cancel_20x20_xpm
	);
	gtk_box_pack_start(GTK_BOX(parent), w, FALSE, FALSE, 0);
	gtk_signal_connect(
	    GTK_OBJECT(w), "clicked",
	    GTK_SIGNAL_FUNC(FPromptCancelCB), p
	);

	/* Reset callbacks */
	p->client_data = NULL;
	p->browse_cb = NULL;
	p->apply_cb = NULL;
	p->cancel_cb = NULL;

	return(0);
}

/*
 *	Sets Floating Prompt as a transient for the given window.
 *
 *	This function has no affect since the Floating Prompt's
 *	GtkWindow is a GTK_WINDOW_POPUP and its entry grabs input,
 *	so there is no need to set it modal or transient for.
 */
void FPromptSetTransientFor(GtkWidget *w)
{

}

/*
 *	Returns TRUE if currently blocking for query.
 */
gboolean FPromptIsQuery(void)
{
	fprompt_struct *p = &fprompt;
	GtkWidget *w = (p != NULL) ? p->toplevel : NULL;
	return((w != NULL) ? GTK_WIDGET_MAPPED(w) : FALSE);
}

/*
 *	Ends query if any and returns a not available response.
 */
void FPromptBreakQuery(void)
{
	fprompt_struct *p = &fprompt;

	/* If still in query then call the cancel callback and unmap
	 * the Floating Prompt
	 */
	if(FPromptIsQuery())
	    FPromptCancelCB(NULL, p);

	/* Reset values */
	p->client_data = NULL;
	p->browse_cb = NULL;
	p->apply_cb = NULL;
	p->cancel_cb = NULL;
}

/*
 *	Moves the Floating Prompt to the specified coordinates the
 *	next time it is mapped.
 */
void FPromptSetPosition(gint x, gint y)
{
	fprompt_struct *p = &fprompt;
	GtkWidget *w = (p != NULL) ? p->toplevel : NULL;
	if(w == NULL)
	    return;

	gtk_widget_set_uposition(w, x, y);
}

/*
 *	Maps the Floating Prompt and sets it up with the given values.
 *
 *	Does not block execution.
 *
 *	Returns 0 on success or non-zero on error.
 */
gint FPromptMapQuery(
	const gchar *label,		/* Hides label if NULL */
	const gchar *value,
	const gchar *tooltip_message,
	fprompt_map_code map_code,	/* One of FPROMPT_MAP_* */
	gint width, gint height,
	fprompt_flags flags,		/* Any of FPROMPT_FLAG_* */
	gpointer client_data,
	gchar *(*browse_cb)(gpointer, const gchar *),
	void (*apply_cb)(gpointer, const gchar *),
	void (*cancel_cb)(gpointer)
)
{
	GtkWidget *w;
	fprompt_struct *p = &fprompt;

	if(!p->initialized)
	    return(-1);

	/* Set new label */
	w = p->label;
	if(w != NULL)
	{
	    if(label == NULL)
	    {
		gtk_widget_hide(w);
	    }
	    else
	    {
		gtk_label_set_text(GTK_LABEL(w), label);
		gtk_widget_show(w);
	    }
	}

	/* Set new entry value */
	w = p->entry;
	if(w != NULL)
	{
	    if(value != NULL)
	    {
		gint len = STRLEN(value);
		gtk_entry_set_text(GTK_ENTRY(w), value);
		if(flags & FPROMPT_FLAG_SELECT_VALUE)
		    gtk_entry_select_region(GTK_ENTRY(w), 0, len);
		gtk_entry_set_position(GTK_ENTRY(w), len);
	    }
	    else
	    {
		/* Leave value it was */
	    }

	    /* Set new tooltip message */
	    GUISetWidgetTip(w, tooltip_message);
	}

	/* Show/hide buttons */
	w = p->browse_btn;
	if(w != NULL)
	{
	    if(flags & FPROMPT_FLAG_BROWSE_BTN)
		gtk_widget_show(w);
	    else
		gtk_widget_hide(w);	    
	}

	w = p->ok_btn;
	if(w != NULL)
	{
	    if(flags & FPROMPT_FLAG_OK_BTN)
		gtk_widget_show(w);
	    else
		gtk_widget_hide(w);
	}

	w = p->cancel_btn;
	if(w != NULL)
	{
	    if(flags & FPROMPT_FLAG_CANCEL_BTN)
		gtk_widget_show(w);
	    else
		gtk_widget_hide(w);
	}


	/* Update toplevel */
	w = p->toplevel;
	if(w != NULL)
	{
	    gint x = 0, y = 0;
	    GdkWindow	*window = w->window,
			*root = (window != NULL) ?
			    gdk_window_get_parent(window) : NULL;

	    /* Set size of toplevel */
	    gtk_widget_set_usize(w, width, height);
	    gtk_widget_queue_resize(w);

	    /* Record size of toplevel */
	    p->width = (width > 0) ? width : w->allocation.width;
	    p->height = (height > 0) ? height : w->allocation.height;
	    if(p->width <= 0)
		p->width = FPROMPT_DEF_WIDTH;
	    if(p->height <= 0)
		p->height = FPROMPT_DEF_HEIGHT;

	    /* Move the toplevel's GdkWindow, calculate x and y
	     * position relative to parent (root) GdkWindow by the
	     * specified map code
	     */
	    switch(map_code)
	    {
	      case FPROMPT_MAP_TO_POINTER_WINDOW:
		window = gdk_window_at_pointer(&x, &y);
		if(window != NULL)
		{
		    gint wwidth, wheight;
		    gdk_window_get_root_position(window, &x, &y);
		    gdk_window_get_size(window, &wwidth, &wheight);
		    x += (wwidth / 2) - (p->width / 2);
		    y += (wheight / 2) - (p->height / 2);
		    gtk_widget_set_uposition(w, x, y);
		}
		break;

	      case FPROMPT_MAP_TO_POINTER:
		if(root != NULL)
		{
		    gdk_window_get_pointer(root, &x, &y, 0);
		    x -= (p->width / 2);
		    y -= (p->height / 2);
		}
		gtk_widget_set_uposition(w, x, y);
		break;

	      case FPROMPT_MAP_NO_MOVE:
		break;
	    }
	}

	/* Set callbacks */
	p->client_data = client_data;
	p->browse_cb = browse_cb;
	p->apply_cb = apply_cb;
	p->cancel_cb = cancel_cb;

	/* Map Floating Prompt */
	FPromptMap();

	return(0);
}

/*
 *	Maps the Floating Prompt.
 */
void FPromptMap(void)
{
	fprompt_struct *p = &fprompt;
	GtkWidget *w = p->toplevel;
	if(w == NULL)
	    return;

	gtk_widget_show_raise(w);

	w = p->entry;
	if(w != NULL)
	{
	    gtk_widget_grab_focus(w);
	    gtk_widget_grab_default(w);
	    if(w != gtk_grab_get_current())
		gtk_grab_add(w);
	    GTK_WIDGET_SET_FLAGS(w, GTK_HAS_FOCUS);
	}
}

/*
 *	Unmaps the Floating Prompt.
 */
void FPromptUnmap(void)
{
	fprompt_struct *p = &fprompt;
	GtkWidget *w;

	w = p->entry;
	if((w != NULL) && (w == gtk_grab_get_current()))
	    gtk_grab_remove(w);

	p->width = 0;
	p->height = 0;

	w = p->toplevel;
	if(w != NULL)
	    gtk_widget_hide(w);
}

/*
 *	Shuts down the Floating Prompt.
 */
void FPromptShutdown(void)
{
	fprompt_struct *p = &fprompt;

	if(p->initialized)
	{
	    GTK_WIDGET_DESTROY(p->cancel_btn)
	    GTK_WIDGET_DESTROY(p->ok_btn)
	    GTK_WIDGET_DESTROY(p->browse_btn)
	    GTK_WIDGET_DESTROY(p->entry)
	    GTK_WIDGET_DESTROY(p->label)
	    GTK_WIDGET_DESTROY(p->main_hbox)
	    GTK_WIDGET_DESTROY(p->toplevel)
	}

	/* Clear floating prompt structure */
	memset(p, 0x00, sizeof(fprompt_struct));
}
